UNIT Mathunit;
INTERFACE
VAR
//input given externally
    MAC, U, H, D, dT, M, W0, Xf, A, F:single;
//calculated, stored ...
//first block
    Cm, m2, f2, n, Vm, V1,
//second block
    Xm, d2, Um,
//third block
    Cmn, r,
//fourth block
    Xmn, p,
//fifth & sixth block
    C, s1, Cy, s2:single;
    Hm, MTO:single;

CONST
    calculated='Calculated -> %s = %.4f';


VAR
    dataarea:ARRAY OF single;

//first block
PROCEDURE FRM_Cm;
PROCEDURE FRM_m2;
PROCEDURE FRM_f2;
PROCEDURE FRM_n;
PROCEDURE FRM_Vm;
PROCEDURE FRM_V1;
//second block
PROCEDURE FRM_Xm;
PROCEDURE FRM_d2;
PROCEDURE FRM_Um;
//third
PROCEDURE FRM_Cmn;
PROCEDURE FRM_r;
//fourth
PROCEDURE FRM_Xmn;
PROCEDURE FRM_p;

PROCEDURE FRM_Hm;
PROCEDURE FRM_MTO;


//ensure, that Xmn has been calculated before calling this function
PROCEDURE FRM_Cxy(oldC, x, y:single);
PROCEDURE FRM_C(x:single);

//general
PROCEDURE calc_undependend;
FUNCTION root(base, exponent:extended):extended;

IMPLEMENTATION
USES math,
    mainunit1,
    sysutils,
    aresources;

FUNCTION root(base, exponent:extended):extended;
VAR
    neg, even:boolean;
BEGIN
    neg:=false;
    even:=false;
    IF base<0 THEN BEGIN
        IF trunc(exponent)MOD 2=0 THEN even:=true;
        IF even=false THEN base:=abs(base);
        neg:=true;
    END;
    IF neg AND(even=false) THEN
        root:=-power(base, 1/exponent)
    ELSE
        root:=power(base, 1/exponent);
END;

PROCEDURE debugview(str:STRING);
BEGIN
    form1.debugview(str);
END;

PROCEDURE FRM_Cm;
BEGIN
//
    Cm:=(A*M*F*m2*n)/(sqr(H)*root(V1*dT, 3));
    debugview(format(calculated, ['Cm', Cm]));
END;

PROCEDURE FRM_m2;
BEGIN
//
    m2:=1/(0.67+0.34*root(f2, 3)+0.1*sqrt(f2));
    debugview(format(calculated, ['m2', m2]));
END;

PROCEDURE FRM_f2;
BEGIN
//
    f2:=(1000*sqr(W0)*D)/(sqr(H)*dT);
    debugview(format(calculated, ['f2', f2]));
END;

PROCEDURE FRM_n;
BEGIN
//
    IF Vm<=0.3 THEN n:=3 ELSE
        IF ((Vm>0.3)AND(Vm<=2)) THEN n:=3-sqrt((Vm-0.3)*(4.36-Vm)) ELSE
            IF Vm>2 THEN n:=1;
    debugview(format(calculated, ['n', n]));
END;

PROCEDURE FRM_Vm;
BEGIN
//
    Vm:=0.65*root((V1*dT)/H, 3);
    debugview(format(calculated, ['Vm', Vm]));
END;

PROCEDURE FRM_V1;
BEGIN
//
    V1:=(pi*sqr(D)*W0)/4;
    debugview(format(calculated, ['V1', V1]));
END;

PROCEDURE FRM_Xm;
BEGIN
//
    IF F>=2 THEN Xm:=((5-F)/4)*d2*H ELSE
        Xm:=d2*H;
    debugview(format(calculated, ['Xm', Xm]));
END;

PROCEDURE FRM_d2;
BEGIN
//
    IF Vm<=2 THEN d2:=4.95*Vm*(1+0.28*root(f2, 3)) ELSE
        IF Vm>2 THEN d2:=7*sqrt(Vm)*(1+0.28*root(f2, 3));
    debugview(format(calculated, ['d2', d2]));
END;

PROCEDURE FRM_Um;
BEGIN
//
    IF Vm<=0.5 THEN Um:=0.5 ELSE
        IF ((Vm>0.5)AND(Vm<=2)) THEN Um:=Vm ELSE
            IF Vm>2 THEN Um:=Vm*(1+0.12*root(f2, 3));
    debugview(format(calculated, ['Um', Um]));
END;

PROCEDURE FRM_Cmn;
BEGIN
//
    Cmn:=r*Cm;
    debugview(format(calculated, ['Cmn', Cmn]));
END;

PROCEDURE FRM_r;
BEGIN
//
    IF (U/Um)<=1 THEN r:=0.67*(U/Um)+1.67*sqr(U/Um)-1.34*power(U/Um, 3) ELSE
        IF (U/Um)>1 THEN r:=(3*(U/Um))/(2*sqr(U/Um)-(U/Um)+2);
    debugview(format(calculated, ['r', r]));
END;

PROCEDURE FRM_Xmn;
BEGIN
//
    Xmn:=p*Xm;
    debugview(format(calculated, ['Xmn', Xmn]));
END;

PROCEDURE FRM_p;
BEGIN
//
    IF (U/Um)<=0.25 THEN p:=3 ELSE
        IF ((U/Um>0.25)AND(U/Um<=1)) THEN p:=8.43*power(1-(U/Um), 5)+1 ELSE
            IF (U/Um)>1 THEN p:=0.32*(U/Um)+0.68;
    debugview(format(calculated, ['p', p]));
END;

PROCEDURE FRM_Hm;
BEGIN
    Hm:=sqrt((A*M*F*m2*n)/(MAC*root(V1*dT, 3)));
    debugview(format(calculated, ['Hm', Hm]));
END;

PROCEDURE FRM_MTO;
BEGIN
    MTO:=(MAC*sqr(H)*root(V1*dT, 3))/(A*F*m2*n);
    debugview(format(calculated, ['MTO', MTO]));
END;

PROCEDURE calc_undependend;
BEGIN
//this is the dependency order
    FRM_V1;
    FRM_Vm;
    FRM_n;
    FRM_f2;
    FRM_m2;
    FRM_Cm;
    FRM_d2;
    FRM_Xm;
    FRM_Um;
    FRM_r;
    FRM_Cmn;
    FRM_p;
    FRM_Xmn;
    FRM_Hm;
    FRM_MTO;
END;

PROCEDURE FRM_C(x:single);

    PROCEDURE FRM_s1;
    BEGIN
        IF (x/Xmn)<=1 THEN s1:=3*power(x/Xmn, 4)-8*power(x/Xmn, 3)+6*sqr(x/Xmn) ELSE
            IF ((x/Xmn>1)AND(x/Xmn<=8)) THEN s1:=1.13/(0.13*sqr(x/Xmn)+1) ELSE
                IF (x/Xmn)>8 THEN BEGIN
                    IF F=1 THEN s1:=(x/Xmn)/(3.58*sqr(x/Xmn)-35.2*(x/Xmn)+120) ELSE
                        IF F>=2 THEN s1:=1/(0.1*sqr(x/Xmn)+2.47*(x/Xmn)-17.8);
                END;
    END;
BEGIN
    IF x>0 THEN BEGIN
        FRM_s1;
        C:=s1*Cmn;
    END ELSE C:=0;
END;


PROCEDURE FRM_Cxy(oldC, x, y:single);
//oldc decreases the needed time, since we already
//calculated the C's for the certain X-values

    PROCEDURE FRM_s2;
    BEGIN
        s2:=1/((1+8.4*U*sqr(y/x))*(1+28.2*sqr(U)*power(y/x, 4)));
    END;
BEGIN
    IF y<>0 THEN BEGIN
        FRM_s2;
        Cy:=s2*oldC;
    END ELSE Cy:=0;
END;

INITIALIZATION
    MAC:=0;
    U:=0;
    H:=0;
    D:=0;
    dT:=0;
    M:=0;
    W0:=0;
    Xf:=0;
    A:=0;
    F:=0;
END.
